/**
 * Season Changer
 * Copyright (c) 2024 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

TextureManager@ texture_manager;
MenuMusic@ menu_music;
MenuBackground@ menu_background;

void Main() {
    @texture_manager = TextureManager();
    @menu_music = MenuMusic();
    @menu_background = MenuBackground();

    array<Meta::PluginCoroutine@> coroutines = {
        startnew(CoroutineFunc(function() { menu_music.Update(true); })),
        startnew(CoroutineFunc(function() { menu_background.Update(true); }))
    };

    for (uint i = 0; i < coroutines.Length; ++i) {
        if (coroutines[i].IsRunning()) yield();
    }

    while (true) {
        startnew(CoroutineFunc(menu_music.Loop));
        startnew(CoroutineFunc(menu_background.Loop));

        sleep(1000);
    }
}
