/**
 * Season Changer
 * Copyright (c) 2024 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

class MenuBackground : SeasonalThing {
    MenuBackground() {}

    void Update(bool startup = false) override {
        Season season = GetNonDefaultSeason(setting_menu_background);

        CTrackMania@ app = cast<CTrackMania@>(GetApp());
        MwFastBuffer<CGameUILayer@> ui_layers = app.MenuManager.MenuCustom_CurrentManiaApp.UILayers;

        for (uint i = 0; i < ui_layers.Length; ++i) {
            CGameUILayer@ ui_layer = ui_layers[i];

            string header = ui_layer.ManialinkPageUtf8.SubStr(0, 100);

            if (header.IndexOf("\"Overlay_MenuBackground\"") != -1) {
                CGameManialinkPage@ page = ui_layer.LocalPage;
                CGameManialinkQuad@ quad_old = cast<CGameManialinkQuad@>(page.GetFirstChild("HomeBackground_quad-stadium-old"));
                CGameManialinkQuad@ quad_active = cast<CGameManialinkQuad@>(page.GetFirstChild("HomeBackground_quad-stadium-active"));

                quad_old.ImageUrl = ReplaceUrl(quad_old.ImageUrl, season);
                quad_active.ImageUrl = ReplaceUrl(quad_active.ImageUrl, season);
            }
        }
    }

    void Loop() override {
        CTrackMania@ app = cast<CTrackMania@>(GetApp());

        // menu layers don't have to be checked if a playground exists
        if (app.Network.ClientManiaAppPlayground !is null ||
            app.Editor !is null ||
            app.MenuManager.MenuCustom_CurrentManiaApp is null) return;

        Update();
    }

    private string ReplaceUrl(string &in url, Season season) {
        return Regex::Replace(url, "Winter|Spring|Summer|Fall", tostring(season));
    }
}
