/**
 * Season Changer
 * Copyright (c) 2024 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

class MenuMusic : SeasonalThing {
    CAudioScriptSound@ winter_music = null;
    CAudioScriptSound@ spring_music = null;
    CAudioScriptSound@ summer_music = null;
    CAudioScriptSound@ fall_music = null;

    MenuMusic() {
        CTrackMania@ app = cast<CTrackMania@>(GetApp());
        CAudioScriptManager@ manager = app.MenuManager.MenuCustom_CurrentManiaApp.Audio;

        // string Url, float VolumedB, bool IsMusic, bool IsLooping, bool IsSpatialized
        @winter_music = manager.CreateSoundEx("file://Media/Musics/Stadium/Menu/MainMenu-Winter.ogg",
                                              0.0, true, false, false);
        @spring_music = manager.CreateSoundEx("file://Media/Musics/Stadium/Menu/MainMenu-Spring.ogg",
                                              0.0, true, false, false);
        @summer_music = manager.CreateSoundEx("file://Media/Musics/Stadium/Menu/MainMenu-Summer.ogg",
                                              0.0, true, false, false);
        @fall_music = manager.CreateSoundEx("file://Media/Musics/Stadium/Menu/MainMenu-Fall.ogg",
                                            0.0, true, false, false);
    }

    void Update(bool startup = false) override {
        Season season = GetNonDefaultSeason(setting_menu_music);

        ToggleMusic(season, Season::Winter, winter_music, startup);
        ToggleMusic(season, Season::Spring, spring_music, startup);
        ToggleMusic(season, Season::Summer, summer_music, startup);
        ToggleMusic(season, Season::Fall, fall_music, startup);
    }

    void Loop() override {
        CTrackMania@ app = cast<CTrackMania@>(GetApp());

        // menu layers don't have to be checked if a playground exists
        if (app.Network.ClientManiaAppPlayground !is null ||
            app.Editor !is null ||
            app.MenuManager.MenuCustom_CurrentManiaApp is null) return;

        Update();
    }

    private void ToggleMusic(Season season_to_play, Season season_music, CAudioScriptSound@ music, bool startup) {
        if (music.IsPlaying && season_to_play != season_music) music.Stop();

        if (startup) {
            // when starting the game, it might take a few tries before the music actually plays
            while (!music.IsPlaying && season_to_play == season_music) {
                music.Play();
                sleep(2000);
            }
        } else {
            if (!music.IsPlaying && season_to_play == season_music) music.Play();
        }
    }
}
