/**
 * Season Changer
 * Copyright (c) 2024 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

class SeasonalThing {
    SeasonalThing() {}

    void Update(bool startup = false) {}

    void Loop() {}

    Season RenderSettings(Season setting) final {
        vec2 item_spacing = UI::GetStyleVarVec2(UI::StyleVar::ItemSpacing);
        item_spacing.y = item_spacing.x;
        UI::PushStyleVar(UI::StyleVar::ItemSpacing, item_spacing);

        if (RenderSettingButton(Season::Default, setting)) setting = Season::Default;

        if (RenderSettingButton(Season::Winter, setting)) setting = Season::Winter;
        UI::SameLine();
        if (RenderSettingButton(Season::Spring, setting)) setting = Season::Spring;

        if (RenderSettingButton(Season::Summer, setting)) setting = Season::Summer;
        UI::SameLine();
        if (RenderSettingButton(Season::Fall, setting)) setting = Season::Fall;

        UI::PopStyleVar(1);

        return setting;
    }

    private bool RenderSettingButton(Season setting_button, Season current_setting) final {
        float scale = UI::GetScale();
        vec2 frame_padding = UI::GetStyleVarVec2(UI::StyleVar::FramePadding);
        vec2 item_spacing = UI::GetStyleVarVec2(UI::StyleVar::ItemSpacing);
        vec2 content_region = UI::GetContentRegionMax();
        vec4 text_color = UI::GetStyleColor(UI::Col::Text);

        vec2 string_size = Draw::MeasureString(tostring(setting_button));

        float min_internal_button_width = 16.0 * scale;
        float max_icon_size = 72.0 * scale;
        float max_full_width = 360.0 * scale;

        float min_full_width = 4 * frame_padding.x + item_spacing.x + 2 * min_internal_button_width;

        float full_width = Math::Clamp(content_region.x, min_full_width, max_full_width);
        float button_width = (full_width - item_spacing.x) / 2;
        float icon_size = Math::Clamp(button_width - 2 * frame_padding.x, min_internal_button_width, max_icon_size);

        float button_height = icon_size + string_size.y + 2 * frame_padding.y;

        vec2 button_size;
        if (setting_button == Season::Default) {
            button_size = vec2(full_width, button_height);
        } else {
            button_size = vec2(button_width, button_height);
        }

        vec2 window_pos = UI::GetWindowPos();
        vec2 window_size = UI::GetWindowSize();

        vec2 button_start = window_pos + UI::GetCursorPos();
        vec4 button_color = UI::GetStyleColor((setting_button == current_setting) ? UI::Col::ButtonActive : UI::Col::Button);

        UI::PushStyleColor(UI::Col::Button, button_color);
        bool clicked = UI::Button("##" + tostring(setting_button), button_size);
        UI::PopStyleColor(1);

        UI::DrawList@ draw_list = UI::GetForegroundDrawList();

        draw_list.PushClipRect(vec4(button_start.x, button_start.y, button_size.x, button_size.y), true);
        draw_list.PushClipRect(vec4(window_pos.x, window_pos.y, window_size.x, window_size.y), true);

        vec2 icon_position = vec2(button_start.x + button_size.x / 2 - icon_size / 2,
                                  button_start.y + frame_padding.y);
        draw_list.AddImage(texture_manager.GetTexture(setting_button), icon_position, vec2(icon_size, icon_size));

        vec2 text_position = vec2(button_start.x + button_size.x / 2 - string_size.x / 2,
                                  button_start.y + button_size.y - string_size.y - frame_padding.y);
        draw_list.AddText(text_position, text_color, tostring(setting_button));

        draw_list.PopClipRect();
        draw_list.PopClipRect();

        return clicked;
    }
}
